# generated by datamodel-codegen:
#   filename:  schema/api/data/createLocation.json
#   timestamp: 2022-01-16T19:33:03+00:00

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

from ...entity.data import location
from ...type import entityReference, tagLabel


class CreateLocationEntityRequest(BaseModel):
    name: location.LocationName = Field(
        ..., description='Name that identifies this Location.'
    )
    description: Optional[str] = Field(
        None, description='Description of the location instance.'
    )
    locationType: Optional[location.LocationType] = None
    tags: Optional[List[tagLabel.TagLabel]] = Field(
        None, description='Tags for this location'
    )
    owner: Optional[entityReference.EntityReference] = Field(
        None, description='Owner of this Location'
    )
    service: entityReference.EntityReference = Field(
        ..., description='Link to the pipeline service where this location is used'
    )
