# generated by datamodel-codegen:
#   filename:  schema/api/teams/createRole.json
#   timestamp: 2022-01-16T18:43:22+00:00

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, Field

from ...entity.teams import role


class CreateRoleEntityRequest(BaseModel):
    name: role.RoleName
    displayName: Optional[str] = Field(
        None,
        description="Optional name used for display purposes. Example 'Data Consumer'",
    )
    description: Optional[str] = Field(
        None, description='Optional description of the role'
    )
